@@ -17,7 +17,7 @@
 
 package Git::Hooks::CheckJira;
 {
-  $Git::Hooks::CheckJira::VERSION = '0.040';
+  $Git::Hooks::CheckJira::VERSION = '0.041';
 }
 # ABSTRACT: Git::Hooks plugin which requires citation of JIRA issues in commit messages.
 
@@ -252,6 +252,15 @@ EOF
     return _check_jira_keys($git, $commit, $ref, @keys);
 }
 
+sub check_patchset {
+    my ($git, $opts) = @_;
+
+    my $sha1   = $opts->{'--commit'};
+    my $commit = ($git->get_commits("$sha1^", $sha1))[-1];
+
+    return check_commit_msg($git, $commit, $opts->{'--branch'});
+}
+
 sub check_message_file {
     my ($git, $commit_msg_file) = @_;
 
@@ -314,10 +323,11 @@ sub check_affected_refs {
 }
 
 # Install hooks
-COMMIT_MSG  \&check_message_file;
-UPDATE      \&check_affected_refs;
-PRE_RECEIVE \&check_affected_refs;
-
+COMMIT_MSG       \&check_message_file;
+UPDATE           \&check_affected_refs;
+PRE_RECEIVE      \&check_affected_refs;
+REF_UPDATE       \&check_affected_refs;
+PATCHSET_CREATED \&check_patchset;
 1;
 
 __END__
@@ -330,7 +340,7 @@ Git::Hooks::CheckJira - Git::Hooks plugin which requires citation of JIRA issues
 
 =head1 VERSION
 
-version 0.040
+version 0.041
 
 =head1 DESCRIPTION
 
@@ -357,6 +367,17 @@ message cites valid JIRA issues.
 This hook is invoked once in the remote repository during C<git push>,
 to check if the commit message cites valid JIRA issues.
 
+=item * B<ref-update>
+
+This hook is invoked when a push request is received by Gerrit Code
+Review, to check if the commit message cites valid JIRA issues.
+
+=item * B<patchset-created>
+
+This hook is invoked when a push request is received by Gerrit Code
+Review for a virtual branch (refs/for/*), to check if the commit
+message cites valid JIRA issues.
+
 =back
 
 It requires that any Git commits affecting all or some branches must
@@ -439,8 +460,27 @@ default issue pattern can match other things besides JIRA issue
 keys. You may use this option to restrict the places inside the commit
 message where the keys are going to be looked for.
 
-For example, set it to C<\[([^]]+)\]> to require that JIRA keys be
-cited inside the first pair of brackets found in the message.
+Here are some examples:
+
+=over
+
+=item * C<\[([^]]+)\]>
+
+Looks for JIRA keys inside the first pair of brackets found in the
+message.
+
+=item * C<(?s)^\[([^]]+)\]>
+
+Looks for JIRA keys inside a pair of brackets that must be at the
+beginning of the message's title.
+
+=item * C<(?im)^Bug:(.*)>
+
+Looks for JIRA keys in a line beginning with C<Bug:>. This is a common
+convention around some high caliber projects, such as OpenStack and
+Wikimedia.
+
+=back
 
 =head2 githooks.checkjira.project STRING
 
@@ -525,6 +565,12 @@ This is the routine used to implement the C<commit-msg> hook. It needs
 a C<Git::More> object and the name of a file containing the commit
 message.
 
+=head2 check_patchset GIT, HASH
+
+This is the routine used to implement the C<patchset-created> Gerrit
+hook. It needs a C<Git::More> object and the hash containing the
+arguments passed to the hook by Gerrit.
+
 =head1 SEE ALSO
 
 B<Git::More>
