@@ -17,7 +17,7 @@
 
 package Git::Hooks::CheckLog;
 {
-  $Git::Hooks::CheckLog::VERSION = '0.040';
+  $Git::Hooks::CheckLog::VERSION = '0.041';
 }
 # ABSTRACT: Git::Hooks plugin to enforce commit log policies.
 
@@ -247,6 +247,17 @@ sub check_message {
     return $errors == 0;
 }
 
+sub check_patchset {
+    my ($git, $opts) = @_;
+
+    _setup_config($git);
+
+    my $sha1   = $opts->{'--commit'};
+    my $commit = ($git->get_commits("$sha1^", $sha1))[-1];
+
+    return check_message($git, $commit, $commit->{body});
+}
+
 sub check_message_file {
     my ($git, $commit_msg_file) = @_;
 
@@ -291,9 +302,11 @@ sub check_affected_refs {
 }
 
 # Install hooks
-COMMIT_MSG  \&check_message_file;
-UPDATE      \&check_affected_refs;
-PRE_RECEIVE \&check_affected_refs;
+COMMIT_MSG       \&check_message_file;
+UPDATE           \&check_affected_refs;
+PRE_RECEIVE      \&check_affected_refs;
+REF_UPDATE       \&check_affected_refs;
+PATCHSET_CREATED \&check_patchset;
 
 1;
 
@@ -307,7 +320,7 @@ Git::Hooks::CheckLog - Git::Hooks plugin to enforce commit log policies.
 
 =head1 VERSION
 
-version 0.040
+version 0.041
 
 =head1 DESCRIPTION
 
@@ -333,6 +346,18 @@ This hook is invoked once in the remote repository during C<git push>,
 to check if the commit log messages of all commits being pushed
 comply.
 
+=item * B<ref-update>
+
+This hook is invoked when a push request is received by Gerrit Code
+Review, to check if the commit log messages of all commits being
+pushed comply.
+
+=item * B<patchset-created>
+
+This hook is invoked when a push request is received by Gerrit Code
+Review for a virtual branch (refs/for/*), to check if the commit log
+messages of all commits being pushed comply.
+
 =back
 
 Projects using Git, probably more than projects using any other
@@ -453,6 +478,12 @@ message.
 This is the routing used to implement the C<update> and the
 C<pre-receive> hooks. It needs a C<Git::More> object.
 
+=head2 check_patchset GIT, HASH
+
+This is the routine used to implement the C<patchset-created> Gerrit
+hook. It needs a C<Git::More> object and the hash containing the
+arguments passed to the hook by Gerrit.
+
 =head1 REFERENCES
 
 =over
@@ -484,6 +515,11 @@ L<This is a good
 discussion|http://ablogaboutcode.com/2011/03/23/proper-git-commit-messages-and-an-elegant-git-history/>
 about commit log message formatting and the reasons behind them.
 
+=item * B<GIT Commit Good Practice>
+
+L<This document|https://wiki.openstack.org/wiki/GitCommitMessages>
+defines the OpenStack's project commit policies.
+
 =back
 
 =head1 AUTHOR
